/*
 * gsampler-statusbar.c
 * Copyright (C) Lilian Monchalin 2011 <isaya07@users.sf.net>
 * 
 * gsampler is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * gsampler is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "gsampler-statusbar.h"

G_DEFINE_TYPE (GsamplerStatusBar, gsampler_status_bar, GTK_TYPE_STATUSBAR);

static gboolean 
gsampler_status_bar_mask_progress (gpointer data) 
{
	GsamplerStatusBar* self;
	self = GSAMPLER_STATUS_BAR (data);
	g_return_val_if_fail (self != NULL, FALSE);
	
	gtk_widget_hide ((GtkWidget*) self->priv->progress);
	gtk_progress_bar_set_text (self->priv->progress, "");
	g_source_remove (self->priv->timeout_id);
	return FALSE;
}

static gboolean 
gsampler_status_bar_mask_error_message (gpointer data)
{
	GsamplerStatusBar* self;
	self = GSAMPLER_STATUS_BAR (data);
	g_return_val_if_fail (GSAMPLER_IS_STATUS_BAR (self), FALSE);
	
	gtk_statusbar_pop ((GtkStatusbar*) self, 1);
	return FALSE;
}

static void 
gsampler_status_bar_init (GsamplerStatusBar * self)
{
	self->priv = GSAMPLER_STATUS_BAR_GET_PRIVATE (self);
	self->priv->progress = (GtkProgressBar*) gtk_progress_bar_new ();
#if GTK_CHECK_VERSION (2, 90, 0)
	gtk_progress_bar_set_show_text (self->priv->progress, TRUE);
#endif
	gtk_box_pack_end ((GtkBox*) self, (GtkWidget*) self->priv->progress, FALSE, TRUE, 0);
}


static void 
gsampler_status_bar_finalize (GObject* obj)
{
	G_OBJECT_CLASS (gsampler_status_bar_parent_class)->finalize (obj);
}

static void 
gsampler_status_bar_class_init (GsamplerStatusBarClass * klass) 
{
	gsampler_status_bar_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (GsamplerStatusBarPrivate));
	G_OBJECT_CLASS (klass)->finalize = gsampler_status_bar_finalize;
}

GsamplerStatusBar* 
gsampler_status_bar_new (void) 
{
	return g_object_new (GSAMPLER_TYPE_STATUS_BAR, NULL);
}

void 
gsampler_status_bar_set_error_message (GsamplerStatusBar* self, const gchar* mes) 
{
	GtkBox* box;
	GList* childs;
	GList* child_it;
	g_return_if_fail (self != NULL);
	g_return_if_fail (mes != NULL);

	box = g_object_ref (gtk_statusbar_get_message_area ((GtkStatusbar*) self));
	childs = gtk_container_get_children ((GtkContainer*) box);
	for (child_it = childs; child_it != NULL; child_it = child_it->next) 
	{
		GtkWidget* child;
		child = (GtkWidget*) child_it->data;
		if (GTK_IS_LABEL (child)) 
		{
#if ((GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION < 90) || GTK_MAJOR_VERSION < 2)
			GdkColor red = {0};
			gdk_color_parse ("red", &red);
			gtk_widget_modify_fg (child, GTK_STATE_NORMAL, &red);
#else
			GdkRGBA red = {0};
			gdk_rgba_parse (&red, "red");
			gtk_widget_override_color (child, GTK_STATE_NORMAL, &red);
#endif
		}		
	}
	gtk_statusbar_pop ((GtkStatusbar*) self, 1);
	gtk_statusbar_push ((GtkStatusbar*) self, 1, mes);
	g_timeout_add_full (G_PRIORITY_DEFAULT, 10000, gsampler_status_bar_mask_error_message,
	                    g_object_ref (self), g_object_unref);
	g_list_free (childs);
	g_object_unref (box);
}

void 
gsampler_status_bar_set_progress (GsamplerStatusBar* self, gdouble* perc, const gchar* msg) 
{
	g_return_if_fail (self != NULL);
	g_return_if_fail (msg != NULL);

	if (((gint) (*perc)) == 1 && 
	    gtk_widget_get_visible ((GtkWidget*) self->priv->progress) != FALSE) 
	{
		self->priv->timeout_id = g_timeout_add_full (G_PRIORITY_DEFAULT, (guint) 2000, 
		                                       		 gsampler_status_bar_mask_progress, 
		                                       		 g_object_ref (self), g_object_unref);
	} else if (((gint) (*perc)) == 0 && 
	    		gtk_widget_get_visible ((GtkWidget*) self->priv->progress) != TRUE) 
	{
		gtk_widget_show ((GtkWidget*) self->priv->progress);
	}
	gtk_progress_bar_set_fraction (self->priv->progress, *perc);
	gtk_progress_bar_set_text (self->priv->progress, msg);
	while (gtk_events_pending ()) 
	{
		gtk_main_iteration ();
	}
}

void
gsampler_status_bar_pulse (GsamplerStatusBar* self, gboolean watch)
{
	if (gtk_widget_get_visible ((GtkWidget*) self->priv->progress) != TRUE) 
	{
		gtk_widget_show ((GtkWidget*) self->priv->progress);
	}
	gtk_progress_bar_pulse (self->priv->progress);
	if (!watch) 
	{
		gtk_widget_hide ((GtkWidget*) self->priv->progress);
	}
}